import document from 'document';
import { readFileSync } from 'fs';

var fonts = {};

export function FitFont(_ref) {
  var _this = this;

  var id = _ref.id,
      font = _ref.font,
      halign = _ref.halign,
      valign = _ref.valign,
      letterspacing = _ref.letterspacing;


  this.root = document.getElementById(id);
  this.chars = this.root.getElementsByClassName('fitfont-char');
  this._halign = halign || 'start';
  this._valign = valign || 'baseline';
  this._spacing = letterspacing || 0;

  if (fonts[font] === undefined) {
    try {
      fonts[font] = readFileSync('/mnt/assets/resources/' + font + '/fonts.json', 'json');
    } catch (e) {
      console.error('Font ' + font + ' not found');
      return;
    }
  }
  this.info = fonts[font];

  this.redraw = function () {
    var totalWidth = 0;
    var i = 0;
    var val = _this._text;
    if (val.length > _this.chars.length) {
      console.warn('New element text larger than the number of characters of ' + id);
    }
    while (i < val.length && i < _this.chars.length) {
      var metrics = _this.info.metrics[val[i]];
      if (metrics === undefined) {
        console.error('Char not found ' + val[i] + ' in ' + font);
        _this.chars[i].width = 0;
        _this.chars[i].href = '';
      } else {
        _this.chars[i].width = metrics[0];
        _this.chars[i].height = metrics[1];
        _this.chars[i].x = totalWidth + metrics[2];
        _this.chars[i].y = metrics[3];
        totalWidth += metrics[4];
        if (i < val.length - 1) {
          totalWidth += _this._spacing;
        }
        _this.chars[i].href = font + '/' + val.charCodeAt(i) + '.png';
      }
      i++;
    }
    for (; i < _this.chars.length; i++) {
      _this.chars[i].href = '';

      _this.chars[i].width = 0;
    }

    var offx = 0;
    var offy = 0;
    switch (_this._halign) {
      case 'middle':
        offx -= totalWidth / 2;break;
      case 'end':
        offx -= totalWidth;break;
      case 'start':
      default:
        offx = 0;break;
    }
    switch (_this._valign) {
      case 'top':
        offy = 0;break;
      case 'middle':
        offy -= (_this.info.ascent + _this.info.descent) / 2;break;
      case 'bottom':
        offy -= _this.info.ascent + _this.info.descent;break;
      case 'baseline':
      default:
        offy -= _this.info.ascent;break;
    }

    _this.chars.forEach(function (element) {
      element.x += offx;
      element.y += offy;
    });
  };

  Object.defineProperty(this, 'text', {
    get: function get() {
      return _this._text;
    },
    set: function set(val) {
      _this._text = val;
      _this.redraw();
    }
  });

  Object.defineProperty(this, 'halign', {
    get: function get() {
      return _this._halign;
    },
    set: function set(val) {
      _this._halign = val;
      _this.redraw();
    }
  });

  Object.defineProperty(this, 'valign', {
    get: function get() {
      return _this._valign;
    },
    set: function set(val) {
      _this._valign = val;
      _this.redraw();
    }
  });

  Object.defineProperty(this, 'letterspacing', {
    get: function get() {
      return _this._spacing;
    },
    set: function set(val) {
      _this._spacing = val;
      _this.redraw();
    }
  });
}